import * as echarts from 'echarts'
import 'echarts-gl'
import { setOptions, nowSize } from "./echarts";

let cblx = null

export function setCblx(ele, barData) {
	if (cblx) {
		cblx.dispose()
	}
	let xData = []
	let max = 0

	barData.forEach(i => {
		xData.push(i.label)
		if (i.value > max) {
			max = i.value
		}
	})

	max = Math.round(max * 1.2)

	let maxData = Array(barData.length).fill(max);

	let barWidth = nowSize(15)

	let option = {
		backgroundColor: 'transparent',
		grid: {
			top: '5%',
			left: '2%',
			bottom: '2%',
			right: '5%',
			containLabel: true,
		},
		tooltip: {
			trigger: 'item',
			textStyle: {
				fontSize: nowSize(14)
			}
		},
		animation: false,
		xAxis: [
			{
				show: true,
				type: 'category',
				data: xData,
				axisTick: {
					alignWithLabel: true,
				},
				axisLine: {
					show: false,
				},
				axisLabel: {
					interval: 0,
					textStyle: {
						color: '#fff',
						fontSize: nowSize(13)
					},
					margin: 15,
					formatter: function (value, index) {
						return value.length > 3 ? value.slice(0, 3) + '...' : value;
					}
				},


			},
		],

		yAxis: [{
			min: 0,
			max: max,
			splitNumber: 4,
			interval: Math.round(max / 4),
			splitLine: {
				lineStyle: {
					color: 'rgba(255,255,255, 0.2)',
					type: 'solid'
				}
			},
			axisTick: {
				show: false
			},
			axisLine: {
				show: false,
			},
			axisLabel: {
				textStyle: {
					color: '#fff',
					fontSize: nowSize(12)
				},
			}
		}],
		series: [
			{
				// name: '上部圆',
				type: 'pictorialBar',
				silent: true,
				symbolSize: [nowSize(15), 10],
				symbolOffset: [0, -6],
				symbolPosition: 'end',
				z: 20,
				zlevel: 10,
				data: maxData,
				itemStyle: {
					normal: {
						color: '#0788ff',
						opacity: 0.1,
					},
				}
			},
			{
				// name: '底部圆',
				type: 'pictorialBar',
				silent: true,
				symbolSize: [nowSize(15), 10],
				symbolOffset: [0, 7],
				z: 11,
				// color: '#5BFCF4',
				data: [0, 0, 0, 0, 0],
				itemStyle: {
					color: '#5BFCF4',
				}
			},
			{
				// name: '背景圆柱',
				type: 'bar',
				silent: true,
				barWidth: barWidth,
				barGap: '10%', // Make series be overlap
				barCateGoryGap: '10%',
				z: 16,
				itemStyle: {
					normal: {
						color: '#0788ff',
						opacity: 0.1,
					},
				},
				data: maxData,
			},
			{
				// name: '内部柱子',
				type: 'bar',
				barWidth: barWidth,
				z: 19,
				barGap: '-100%',
				itemStyle: {
					// opacity: 0.7,
					color: function (params) {
						return new echarts.graphic.LinearGradient(0, 0, 0, 1,
							[
								{
									offset: 0,
									color: 'rgba(30, 115, 224, 1)', // 0% 处的颜色
								},
								{
									offset: 1,
									color: 'rgba(27, 203, 247, 1)', // 100% 处的颜色
								},
							],
							false
						);
					},
				},
				label: {
					// 柱图头部显示值
					show: true,
					position: "top",
					distance: nowSize(10),
					color: "#fff",
					fontSize: nowSize(14),
					// fontFamily: 'DIN-Bold',
					fontWeight: 'bold',
					formatter: (params) => {
						return params.value[params.encode.x[0]];
					},
				},
				data: barData,
			},
			{
				// name: '上部圆',
				type: 'pictorialBar',
				silent: true,
				symbolSize: [nowSize(15), 10],
				symbolOffset: [0, -6],
				symbolPosition: 'end',
				z: 22,
				color: '#12B0ED',
				data: barData,
			},
			{
				// name: '底部圆',
				type: 'pictorialBar',
				silent: true,
				symbolSize: [nowSize(15), 10],
				symbolOffset: [0, 7],
				z: 21,
				itemStyle: {
					color: 'rgb(0,134,234)'
				},
				data: barData,
			},
		],
	};
	cblx = setOptions(ele, option)
}

//船舶船籍结构

let cbcjjg = null
export function setCbcjjg(ele, chartData) {
	if (cbcjjg) {
		cbcjjg.dispose()
	}

	const valueBase = 250 / chartData.reduce((pre, i) => Math.max(pre, i.value), 0)
	let baseHeight = Math.round(15 / valueBase)
	chartData.forEach(i => {
		i.value += baseHeight
	})


	// 生成扇形的曲面参数方程，用于 series-surface.parametricEquation
	function getParametricEquation(startRatio, endRatio, isSelected, isHovered, k, height) {

		// 计算
		let midRatio = (startRatio + endRatio) / 2;

		let startRadian = startRatio * Math.PI * 2;
		let endRadian = endRatio * Math.PI * 2;
		let midRadian = midRatio * Math.PI * 2;

		// 如果只有一个扇形，则不实现选中效果。
		if (startRatio === 0 && endRatio === 1) {
			isSelected = false;
		}

		// 通过扇形内径/外径的值，换算出辅助参数 k（默认值 1/3）
		k = typeof k !== 'undefined' ? k : 1 / 3;

		// 计算选中效果分别在 x 轴、y 轴方向上的位移（未选中，则位移均为 0）
		let offsetX = isSelected ? Math.cos(midRadian) * 0.1 : 0;
		let offsetY = isSelected ? Math.sin(midRadian) * 1 : -1;

		// 计算高亮效果的放大比例（未高亮，则比例为 1）
		let hoverRate = isHovered ? 1.05 : 2;

		// 返回曲面参数方程
		return {

			u: {
				min: -Math.PI,
				max: Math.PI * 3,
				step: Math.PI / 32
			},

			v: {
				min: 0,
				max: Math.PI * 2,
				step: Math.PI / 20
			},

			x: function (u, v) {
				if (u < startRadian) {
					return offsetX + Math.cos(startRadian) * (1 + Math.cos(v) * k) * hoverRate;
				}
				if (u > endRadian) {
					return offsetX + Math.cos(endRadian) * (1 + Math.cos(v) * k) * hoverRate;
				}
				return offsetX + Math.cos(u) * (1 + Math.cos(v) * k) * hoverRate;
			},

			y: function (u, v) {
				if (u < startRadian) {
					return offsetY + Math.sin(startRadian) * (1 + Math.cos(v) * k) * hoverRate;
				}
				if (u > endRadian) {
					return offsetY + Math.sin(endRadian) * (1 + Math.cos(v) * k) * hoverRate;
				}
				return offsetY + Math.sin(u) * (1 + Math.cos(v) * k) * hoverRate;
			},

			z: function (u, v) {
				if (u < - Math.PI * 0.5) {
					return Math.sin(u);
				}
				if (u > Math.PI * 2.5) {
					return Math.sin(u);
				}
				return Math.sin(v) > 0 ? 1 * height : -1;
			}
		};
	}


	// 生成模拟 3D 饼图的配置项
	function getPie3D(pieData, internalDiameterRatio) {

		let series = [];
		let sumValue = 0;
		let startValue = 0;
		let endValue = 0;
		let legendData = [];
		let k = typeof internalDiameterRatio !== 'undefined' ? (1 - internalDiameterRatio) / (1 + internalDiameterRatio) : 1 / 3;

		// 为每一个饼图数据，生成一个 series-surface 配置
		for (let i = 0; i < pieData.length; i++) {

			sumValue += pieData[i].value;

			let seriesItem = {
				name: typeof pieData[i].name === 'undefined' ? `series${i}` : pieData[i].name,
				type: 'surface',
				parametric: true,
				wireframe: {
					show: false
				},
				pieData: pieData[i],
				pieStatus: {
					selected: false,
					hovered: false,
					k: k
				}
			};

			if (typeof pieData[i].itemStyle != 'undefined') {

				let itemStyle = {};

				typeof pieData[i].itemStyle.color != 'undefined' ? itemStyle.color = pieData[i].itemStyle.color : null;
				typeof pieData[i].itemStyle.opacity != 'undefined' ? itemStyle.opacity = pieData[i].itemStyle.opacity : null;

				seriesItem.itemStyle = itemStyle;
			}
			series.push(seriesItem);
		}

		// 使用上一次遍历时，计算出的数据和 sumValue，调用 getParametricEquation 函数，
		// 向每个 series-surface 传入不同的参数方程 series-surface.parametricEquation，也就是实现每一个扇形。
		for (let i = 0; i < series.length; i++) {
			endValue = startValue + series[i].pieData.value;
			// console.log(series[i]);
			series[i].pieData.startRatio = startValue / sumValue;
			series[i].pieData.endRatio = endValue / sumValue;
			// series[i].parametricEquation = getParametricEquation(series[i].pieData.startRatio, series[i].pieData.endRatio, false, false, k, series[i].pieData.value);

			// 每个扇形角度等分
			series[i].parametricEquation = getParametricEquation(i / series.length, (i + 1) / series.length, false, false, k, series[i].pieData.value);

			startValue = endValue;

			legendData.push(series[i].name);
		}


		// 添加底座
		series.push({
			silent: true,
			type: "surface",
			parametric: true,
			wireframe: {
				show: false,
			},
			itemStyle: {
				color: 'rgba(2, 61, 143, 1)',
			},
			parametricEquation: getParametricEquation(0, 1, false, false, 0.45, baseHeight)
		});


		// 准备待返回的配置项，把准备好的 legendData、series 传入。
		let option = {
			grid: {
				top: '0%',
				bottom: '0%'
			},
			tooltip: {
				formatter: params => {
					if (params.seriesName !== 'mouseoutSeries') {
						return `${params.seriesName}<br/><span style="display:inline-block;margin-right:5px;border-radius:10px;width:10px;height:10px;background-color:${params.color};"></span>${option.series[params.seriesIndex].pieData.value - baseHeight}`;
					}
				},
				textStyle: {
					fontSize: nowSize(14)
				}
			},
			legend: {
				icon: "circle",
				itemGap: nowSize(40),
				itemHeight: nowSize(10),
				itemWidth: nowSize(10),
				orient: 'horizontal',
				bottom: '0',
				textStyle: {
					fontSize: nowSize(14),
					color: '#fff'
				},
			},
			xAxis3D: {
				min: -1,
				max: 1
			},
			yAxis3D: {
				min: -1,
				max: 1
			},
			zAxis3D: {
				min: -1,
				max: 1
			},
			grid3D: {
				show: false,
				// boxHeight: 1,
				boxHeight: valueBase,
				//top: '30%',
				bottom: '0%',
				// environment: '#021041',
				viewControl: {
					distance: 500,
					alpha: 20,
					beta: 0,
				},

			},
			series: series
		};
		return option;
	}

	// 传入数据生成 option
	let option = getPie3D([
		{
			name: chartData[0].name,
			value: chartData[0].value,
			itemStyle: {
				opacity: 0.5,
				color: 'rgba(1, 121, 207, 0.9)',
			}
		},
		{
			name: chartData[1].name,
			value: chartData[1].value,
			itemStyle: {
				opacity: 0.5,
				color: 'rgb(86,255,187, .9)',
			}
		},
		{
			name: chartData[2].name,
			value: chartData[2].value,
			itemStyle: {
				opacity: 0.5,
				color: 'rgba(209,126,23, .9)',
			}
		}

	], 1.5);


	cbcjjg = setOptions(ele, option)
}

let waterLevel = null
export function setWaterLevel(ele, barData) {
	if(waterLevel) {
		waterLevel.dispose()
	}

	let maxData = [1000];
	let barWidth = 90

	let option = {
		background: 'transparent',
		grid: {
			top: '11%',
			left: '0%',
			bottom: '4%',
			right: '0%',
		},
		tooltip: {
			show: false,
		},
		animation: false,
		xAxis: [
			{
				show: false,
				type: 'category',
				data: [0],
				axisLine: {
					show: false,
				},
				axisLabel: {
					show: false
				},


			},
		],

		yAxis: [{
			show: false,
		}],
		series: [
			{
				// name: '上部圆',
				type: 'pictorialBar',
				silent: true,
				symbolSize: [barWidth, 20],
				symbolOffset: [0, -10],
				symbolPosition: 'end',
				z: 20,
				zlevel: 10,
				data: maxData,
				itemStyle: {
					normal: {
						color: 'rgba(16, 89, 159, 1)',
					},
				}
			},
			{
				// name: '背景圆柱',
				type: 'bar',
				silent: true,
				barWidth: barWidth,
				barGap: '10%', // Make series be overlap
				barCateGoryGap: '10%',
				z: 16,
				itemStyle: {
					normal: {
						color: 'rgba(19, 73, 112, 1)'
					},
				},
				data: maxData,
			},
			{
				// name: '内部柱子',
				type: 'bar',
				silent: true,
				barWidth: barWidth,
				z: 19,
				barGap: '-100%',
				itemStyle: {
					// opacity: 0.7,
					color: function (params) {
						return new echarts.graphic.LinearGradient(0, 0, 0, 1,
							[
								{
									offset: 0,
									color: 'rgba(8, 78, 178, 1)'
								},
								{
									offset: 1,
									color: 'rgba(14, 154, 209, 1)',
								},
							],
							false
						);
					},
				},
				data: barData,
			},
			{
				// name: '上部圆',
				type: 'pictorialBar',
				silent: true,
				symbolSize: [barWidth, 20],
				symbolOffset: [0, -6],
				symbolPosition: 'end',
				z: 22,
				itemStyle: {
					normal: {
						color: function (params) {
							return new echarts.graphic.LinearGradient(0, 0, 0, 1,
								[
									{
										offset: 0,
										color: 'rgba(14, 154, 209, 1)',
									},
									{
										offset: 1,
										color: 'rgba(2, 81, 209, 1)'
									},

								],
								false
							);
						},
					},
				},
				data: barData,
			},
			{
				// name: '底部圆',
				type: 'pictorialBar',
				silent: true,
				symbolSize: [barWidth, 20],
				symbolOffset: [0, 10],
				z: 21,
				itemStyle: {
					color: function (params) {
						return new echarts.graphic.LinearGradient(0, 0, 0, 1,
							[
								{
									offset: 0,
									color: 'rgba(0, 166, 253, 1)',
								},
								{
									offset: 1,
									color: 'rgba(1, 75, 200, 1)'
								},

							],
							false
						);
					},
				},
				data: barData,
			},
		],
	};
	waterLevel = setOptions(ele, option)
}